#!/usr/bin/env python3
from contextlib import contextmanager


class GraphDatabase:

    def __init__(self, storage):
        self._storage = storage

    def close(self):
        self._storage.close()

    def txn(self):
        return self._storage._txn()

    @contextmanager
    def transaction(self):
        txn = self.txn()
        try:
            yield txn
        except Exception as exc:
            txn.rollback()
            raise exc
        else:
            txn.commit()
