#!/usr/bin/env python3
from multiprocessing import Process
from multiprocessing.connection import Listener

from . import GraphDatabase
from .storage import Storage
from .uzelmumu import run


def process(pipe):
    storage = Storage('./_db')
    database = GraphDatabase(storage)
    print('process started')
    while True:
        queries = pipe.recv()
        print('receive queries {}'.format(queries))
        try:
            objects = run(database, *queries)
        except Exception as exc:
            print('error:', exc)
        else:
            pipe.send(objects)
    pipe.close()
    database.close()


if __name__ == '__main__':
    address = ('localhost', 6002)     # family is deduced to be 'AF_INET'

    storage = Storage('./_db')
    database = GraphDatabase(storage)
    with database.transaction() as txn:
        amirouche = txn.vertex.create('user')
        amirouche.props['name'] = 'amirouche'

    # create feed
    with database.transaction() as txn:
        amirouche = txn.vertex.get(amirouche.identifier())

        hypermove = txn.vertex.create('feed')
        hypermove.props['url'] = 'http://hyperdev.fr/atom.xml'
        txn.edge.create(amirouche, 'feeds', hypermove)

        hypermove = txn.vertex.create('feed')
        url = 'http://www.hyperdev.fr/keyword/fran%C3%A7ais/atom.xml'
        hypermove.props['url'] = url
        txn.edge.create(amirouche, 'feeds', hypermove)
    database.close()

    print('id:', amirouche.identifier())

    from multiprocessing import Pipe

    start, end = Pipe(True)
    p = Process(target=process, args=(end,))
    p.start()

    print('running at {}'.format(address))
    with Listener(address) as listener:
        try:
            while True:
                with listener.accept() as cnx:
                    print('received cnx')
                    query = cnx.recv()
                    print('receive query', query[0])
                    start.send(query)
                    print('waiting response')
                    response = start.recv()
                    print('sending response')
                    cnx.send(response)
        except KeyboardInterrupt:
            listener.close()
            print('exiting')
