from msgpack import dumps as msgpack_dumps
from msgpack import loads as msgpack_loads


def dumps(data):
    return msgpack_dumps(data, encoding='utf-8')


def loads(data):
    return msgpack_loads(data, encoding='utf-8')


class LazySliceGenerator:

    def __init__(self, wrapper, generator):
        # FIXME: no need for the wrapper, storage must return only identifiers
        # the manager will use ``storage.*_get`` method when it needs it.
        self._wrapper = wrapper
        self._generator = generator

    def slice(self, start, end):
        """Yield objects starting at index ``start`` until ``end``

        The object at index ``index`` is not part of the slice"""
        index = 0
        while True:
            item = next(self._generator)
            if index >= start and index < end:
                yield self._wrapper(item)
            index += 1

    def first(self):
        """Return the first object"""
        return self.wrapper(next(self.generator()))

    def __iter__(self):
        return self.all()

    def all(self):
        """Return all objects"""
        yield from map(self._wrapper, self._generator)

    def count(self):
        """Return count of objects without evaluating the wrapper function"""
        return len(list(self._generator))
